@@ -33,18 +33,15 @@ _DPiNETWORK_BASE_ADDRESS = 0x24
 
 
 class DPiButton:
-    #
-    # attributes local to this class
-    #
-    _slaveAddress = _DPiNETWORK_BASE_ADDRESS
-    _commErrorCount = 0
-
-
     #
     # constructor for the DPiButton class
     #
     def __init__(self):
-        pass
+        #
+        # attribute variables local to one instance of this class
+        #
+        self._slaveAddress = _DPiNETWORK_BASE_ADDRESS
+        self._commErrorCount = 0
        
 
     # ---------------------------------------------------------------------------------
