@@ -161,34 +161,24 @@ _STEPPER_STATUS_AT_HOME_SWITCH = 0x04
 # other constants used by this class
 #
 _NUMBER_OF_DPi_STEPPER_DRIVERS = 3
-_DPiNETWORK_TIMEOUT_PERIOD_MS = 6
+_DPiNETWORK_TIMEOUT_PERIOD_MS = 8                # 3/9/2023 changed from 6 to 8ms
 _DPiNETWORK_BASE_ADDRESS = 0x20
 
 
 
 class DPiStepper:
-    #
-    # audio wave shapes
-    #
-    SINE_WAVE       = 0
-    TRIANGLE_WAVE   = 1
-    SQUARE_WAVE     = 2
-
-    #
-    # attributes local to this class
-    #
-    _slaveAddress = _DPiNETWORK_BASE_ADDRESS
-    _commErrorCount = 0
-    _stepsPerMillimeter = array('d', [200.0, 200.0, 200.0])
-    _stepsPerRevolution = array('d', [1600.0, 1600.0, 1600.0])
-
-
     #
     # constructor for the DPiStepper class
     #
     def __init__(self):
-        pass
-       
+        #
+        # attribute variables local to one instance of this class
+        #
+        self._slaveAddress = _DPiNETWORK_BASE_ADDRESS
+        self._commErrorCount = 0
+        self._stepsPerMillimeter = array('d', [200.0, 200.0, 200.0])
+        self._stepsPerRevolution = array('d', [1600.0, 1600.0, 1600.0])
+    
 
     # ---------------------------------------------------------------------------------
     #                                 Private functions 
